@@ -14,7 +14,9 @@ mod genesis;
 pub mod melvm;
 mod stake;
 mod state;
+mod units;
 pub use state::melmint::*;
+pub use units::*;
 mod transaction;
 pub use constants::*;
 mod smtmapping;
