@@ -1,6 +1,6 @@
 pub use crate::stake::*;
 use crate::{constants::*, melvm::Address, preseal_melmint, CoinDataHeight, Denom, TxHash};
-use crate::{smtmapping::*, CoinData};
+use crate::{smtmapping::*, BlockHeight, CoinData, CoinValue};
 use crate::{transaction as txn, CoinID};
 use applytx::StateHandle;
 use defmac::defmac;
@@ -33,7 +33,7 @@ pub enum StateError {
     #[error("unbalanced inputs and outputs")]
     UnbalancedInOut,
     #[error("insufficient fees (requires {0})")]
-    InsufficientFees(u128),
+    InsufficientFees(CoinValue),
     #[error("referenced non-existent script {:?}", .0)]
     NonexistentScript(Address),
     #[error("does not satisfy script {:?}", .0)]
@@ -67,6 +67,13 @@ pub enum StateError {
 #[repr(u8)]
 pub enum NetID {
     Testnet = 0x01,
+    Custom02 = 0x02,
+    Custom03 = 0x03,
+    Custom04 = 0x04,
+    Custom05 = 0x05,
+    Custom06 = 0x06,
+    Custom07 = 0x07,
+    Custom08 = 0x08,
     Mainnet = 0xff,
 }
 
@@ -75,14 +82,14 @@ pub enum NetID {
 pub struct State {
     pub network: NetID,
 
-    pub height: u64,
-    pub history: SmtMapping<u64, Header>,
+    pub height: BlockHeight,
+    pub history: SmtMapping<BlockHeight, Header>,
     pub coins: SmtMapping<CoinID, CoinDataHeight>,
     pub transactions: SmtMapping<TxHash, Transaction>,
 
-    pub fee_pool: u128,
+    pub fee_pool: CoinValue,
     pub fee_multiplier: u128,
-    pub tips: u128,
+    pub tips: CoinValue,
 
     pub dosc_speed: u128,
     pub pools: PoolMapping,
@@ -101,14 +108,14 @@ impl State {
     pub fn partial_encoding(&self) -> Vec<u8> {
         let mut out = Vec::new();
         out.extend_from_slice(&[self.network.into()]);
-        out.extend_from_slice(&self.height.to_be_bytes());
+        out.extend_from_slice(&self.height.0.to_be_bytes());
         out.extend_from_slice(&self.history.root_hash());
         out.extend_from_slice(&self.coins.root_hash());
         out.extend_from_slice(&self.transactions.root_hash());
 
-        out.extend_from_slice(&self.fee_pool.to_be_bytes());
+        out.extend_from_slice(&self.fee_pool.0.to_be_bytes());
         out.extend_from_slice(&self.fee_multiplier.to_be_bytes());
-        out.extend_from_slice(&self.tips.to_be_bytes());
+        out.extend_from_slice(&self.tips.0.to_be_bytes());
 
         out.extend_from_slice(&self.dosc_speed.to_be_bytes());
         out.extend_from_slice(&self.pools.root_hash());
@@ -141,14 +148,14 @@ impl State {
         let stakes = readtree!();
         State {
             network,
-            height,
+            height: height.into(),
             history,
             coins,
             transactions,
 
-            fee_pool,
+            fee_pool: fee_pool.into(),
             fee_multiplier,
-            tips,
+            tips: tips.into(),
 
             dosc_speed: dosc_multiplier,
             pools,
@@ -212,10 +219,10 @@ impl State {
             }
 
             // then it's time to collect the fees dude! we synthesize a coin with 1/65536 of the fee pool and all the tips.
-            let base_fees = self.fee_pool >> 16;
+            let base_fees = CoinValue(self.fee_pool.0 >> 16);
             self.fee_pool -= base_fees;
             let tips = self.tips;
-            self.tips = 0;
+            self.tips = 0.into();
             let pseudocoin_id = CoinID::proposer_reward(self.height);
             let pseudocoin_data = CoinDataHeight {
                 coin_data: CoinData {
@@ -273,8 +280,8 @@ impl SealedState {
         // panic!()
         Header {
             network: inner.network,
-            previous: (inner.height.checked_sub(1))
-                .map(|height| inner.history.get(&height).0.unwrap().hash())
+            previous: (inner.height.0.checked_sub(1))
+                .map(|height| inner.history.get(&BlockHeight(height)).0.unwrap().hash())
                 .unwrap_or_default(),
             height: inner.height,
             history_hash: inner.history.root_hash(),
@@ -314,8 +321,8 @@ impl SealedState {
         let mut new = self.inner_ref().clone();
         // fee variables
         new.history.insert(self.0.height, self.header());
-        new.height += 1;
-        new.stakes.remove_stale(new.height / STAKE_EPOCH);
+        new.height += BlockHeight(1);
+        new.stakes.remove_stale((new.height / STAKE_EPOCH).0);
         new.transactions.clear();
         new
     }
@@ -395,11 +402,11 @@ impl ConfirmedState {
 pub struct Header {
     pub network: NetID,
     pub previous: HashVal,
-    pub height: u64,
+    pub height: BlockHeight,
     pub history_hash: HashVal,
     pub coins_hash: HashVal,
     pub transactions_hash: HashVal,
-    pub fee_pool: u128,
+    pub fee_pool: CoinValue,
     pub fee_multiplier: u128,
     pub dosc_speed: u128,
     pub pools_hash: HashVal,
@@ -410,18 +417,6 @@ impl Header {
     pub fn hash(&self) -> tmelcrypt::HashVal {
         tmelcrypt::hash_single(&stdcode::serialize(self).unwrap())
     }
-
-    pub fn validate_cproof(
-        &self,
-        _cproof: &ConsensusProof,
-        previous_state: Option<&State>,
-    ) -> bool {
-        if previous_state.is_none() && self.height != 0 {
-            return false;
-        }
-        // TODO
-        true
-    }
 }
 
 #[derive(Serialize, Deserialize, Clone, Debug)]
